'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

exports.default = initAppManager;

var _appManagerError = require('./utils/app-manager-error');

var _appManagerError2 = _interopRequireDefault(_appManagerError);

var _config = require('./utils/config');

var _shimHistory = require('./utils/shim-history');

var _shimHistory2 = _interopRequireDefault(_shimHistory);

var _reconcile = require('./utils/reconcile');

var _reconcile2 = _interopRequireDefault(_reconcile);

var _lifecycle = require('./utils/lifecycle');

var _lifecycle2 = _interopRequireDefault(_lifecycle);

var _state = require('./utils/state');

var _state2 = _interopRequireDefault(_state);

var _script = require('./utils/script');

var _script2 = _interopRequireDefault(_script);

var _constants = require('./constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function initAppManager(w) {
  if (!w.document || !w.location || !w.history) {
    throw new _appManagerError2.default('Invalid window object. Aborting.', {
      source: 'init',
      level: _constants.levels.ERROR,
      fragment: null,
      slot: null,
      code: 'invalid_environment',
      recoverable: false
    });
  }

  function selectFromDom(querySelector) {
    return w.document.querySelector(querySelector);
  }

  function getCurrentResource() {
    var _w$location = w.location,
        pathname = _w$location.pathname,
        search = _w$location.search;

    return '' + pathname + (search || '');
  }

  function appManager(config, events, overrideOptions) {
    var getSlotElement = function () {
      var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(slotName, fragmentName) {
        var errData, slot, querySelector, element;
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                errData = {
                  source: 'get_slot_element',
                  fragment: fragmentName,
                  slot: slotName
                };
                slot = slots[slotName];

                if (slot) {
                  _context.next = 4;
                  break;
                }

                throw new _appManagerError2.default('Slot ' + slotName + ' could not be found.', (0, _assign2.default)({}, errData, {
                  level: _constants.levels.ERROR,
                  code: 'slot_not_found',
                  recoverable: false
                }));

              case 4:
                querySelector = slot.querySelector;

                if (querySelector) {
                  _context.next = 7;
                  break;
                }

                return _context.abrupt('return', null);

              case 7:
                _context.next = 9;
                return (0, _config.retry)(function () {
                  return selectFromDom(querySelector);
                }, options.domCheckInterval, options.importTimeout);

              case 9:
                element = _context.sent;

                if (element) {
                  _context.next = 12;
                  break;
                }

                throw new _appManagerError2.default('Element ' + querySelector + ' not found for slot ' + slotName + '.', (0, _assign2.default)({}, errData, {
                  level: _constants.levels.ERROR,
                  code: 'element_not_found',
                  recoverable: false
                }));

              case 12:
                return _context.abrupt('return', element);

              case 13:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      return function getSlotElement(_x, _x2) {
        return _ref.apply(this, arguments);
      };
    }();

    var stateChanger = function () {
      var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(eventTitle) {
        var resource, currentApp, newState, initialRender, pageDiff, haveScripts, nextStateChange;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _context2.prev = 0;
                resource = getCurrentResource();
                currentApp = workingState && workingState.app ? workingState.app : null;
                newState = updateState(resource, currentApp, eventTitle);

                if (newState) {
                  _context2.next = 8;
                  break;
                }

                events.emit(_constants.eventTitles.EXTERNAL_LINK);
                runningStateChange = null;
                return _context2.abrupt('return');

              case 8:

                workingState = newState;

                events.emit(_constants.eventTitles.STATUS_CHANGE, (0, _config.getLoadingStatusDetails)());

                initialRender = !workingState.prevApp;
                pageDiff = reconcile(workingState, initialRender);
                _context2.next = 14;
                return lifecycle(pageDiff, initialRender);

              case 14:
                haveScripts = _context2.sent;


                events.emit(_constants.eventTitles.STATUS_CHANGE, (0, _config.getDefaultStatusDetails)());

                if (!(!haveScripts && !initialRender)) {
                  _context2.next = 20;
                  break;
                }

                events.emit(_constants.eventTitles.MISSING_SCRIPTS);
                runningStateChange = null;
                return _context2.abrupt('return');

              case 20:
                _context2.next = 25;
                break;

              case 22:
                _context2.prev = 22;
                _context2.t0 = _context2['catch'](0);

                options.onError(_context2.t0);

              case 25:
                nextStateChange = queuedStateChange;

                if (nextStateChange) {
                  _context2.next = 29;
                  break;
                }

                runningStateChange = null;
                return _context2.abrupt('return');

              case 29:

                queuedStateChange = null;
                runningStateChange = stateChanger(nextStateChange);

              case 31:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this, [[0, 22]]);
      }));

      return function stateChanger(_x3) {
        return _ref2.apply(this, arguments);
      };
    }();

    var runningStateChange = null;
    var queuedStateChange = null;
    var workingState = void 0;

    if (!events || typeof events.emit !== 'function' || typeof events.on !== 'function' || typeof events.removeListener !== 'function') {
      throw new _appManagerError2.default('Invalid events object. Aborting.', {
        source: 'app_manager',
        level: _constants.levels.ERROR,
        fragment: null,
        slot: null,
        code: 'invalid_events',
        recoverable: false
      });
    }

    var defaultOptions = {
      importTimeout: _constants.IMPORT_TIMEOUT,
      domCheckInterval: _constants.DOM_CHECK_INTERVAL,
      getAppNameFromResource: (0, _config.defaultGetAppNameFromResource)(config.apps),
      onError: function onError(err) {
        events.emit(_constants.eventTitles.ERROR, err);
        events.emit(_constants.eventTitles.STATUS_CHANGE, (0, _config.getErrorStatusDetails)(err));
      }
    };

    var options = (0, _assign2.default)({}, defaultOptions, overrideOptions || {});

    var slots = (0, _config.setNames)(config.slots);
    var fragments = (0, _config.setNames)(config.fragments);
    var apps = (0, _config.setNames)(config.apps);

    function getState() {
      return workingState;
    }

    var reconcile = (0, _reconcile2.default)(config);
    var loadScript = (0, _script2.default)(w, fragments, options, getState);
    var lifecycle = (0, _lifecycle2.default)(fragments, slots, loadScript, getSlotElement, options);
    var updateState = (0, _state2.default)(apps, options);

    function handleStateChange(eventTitle) {
      if (runningStateChange) {
        queuedStateChange = eventTitle;
      } else {
        runningStateChange = stateChanger(eventTitle);
      }
    }

    (0, _shimHistory2.default)(w, events, handleStateChange);

    handleStateChange();

    return {
      getState: getState,
      getRunningStateChange: function getRunningStateChange() {
        return runningStateChange;
      }
    };
  }

  appManager.statuses = _constants.statuses;
  appManager.eventTitles = _constants.eventTitles;
  appManager.levels = _constants.levels;

  return appManager;
}